// coding: utf-8
/*! Created by: Udo Schmal | https://www.gocher.me/ */
(function() {
  'use strict';

// initialize App object
function App () {
  /** btn destination element
   * @private
   * @type {DomNode}
   */
  this.btnDest = null;
  /** btn show destination element
   * @private
   * @type {DomNode}
   */
  this.btnToggleDest = null;
  /** background element
   * @private
   * @type {DomNode}
   */
  this.bg = document.getElementById('page-container');
  /** login/logoff btn element
   * @private
   * @type {DomNode}
   */
  this.btn = null;
  /** contact btn element
   * @private
   * @type {DomNode}
   */
  this.contactbtn = null;
  /** dlg element
   * @private
   * @type {DomNode}
   */
  this.dlg = null;
  /** username element
   * @private
   * @type {DomNode}
   */
  this.username = null;
  /** password element
   * @private
   * @type {DomNode}
   */
  this.password = null;

  let language = document.documentElement.lang || 'en';
  /** translations
   * @private
   * @type {Object}
   */
  switch (language.substr(0,2)) {
    case 'de':
      this.translations = {
        'username': 'Benutzername',
        'password': 'Passwort',
        'login': 'Anmelden',
        'logoff': 'Abmelden',
        'contact': 'Kontakt',
        'subject': 'Betreff',
        'message': 'Nachricht',
        'submit': 'Abschicken',
        'msg1': 'Bitte beachtet, dass Ihr in der Nachricht eine Kontaktmöglichkeit angebt, falls Ihr eine Kontaktaufnahme wünscht.',
        'msg2': 'Aus Datenschutzgründen wird nun Name oder E-Mail-Adresse nicht mehr abgefragt!',
        'users': 'Benutzer',
        'analysis': 'Logfile-Analyse',
        'dropbox': 'Dateien hochladen',
        'check modifications': 'Server Datei-Optimierung',
        'edit message: Page is protected': 'Meldung Geschützte Seite bearbeiten',
        'edit message: Page not valid': 'Meldung Seite ungülig bearbeiten',
        'edit message: Page not found': 'Meldung Seite nicht gefunden bearbeiten',
        'edit mail imprint': 'E-Mail Impressum bearbeiten',
        'edit': 'Bearbeiten',
        'required': 'Bitte füllen Sie das Feld aus!',
        'top': 'an den Anfang',
        'up': 'hoch',
        'down': 'runter',
        'bottom': 'an das Ende',
        'add': 'Hinzufügen',
        'delete': 'Löschen',
        'move': 'Verschieben',
        'exit': 'Abbrechen',
        'fail': 'Fehlgeschlagen',
        'show functions': 'Funktionen anzeigen',
        'hide functions': 'Funktionen ausblenden'
      };
      break;
    default:
      this.translations = {
        'username': 'Username',
        'password': 'Password',
        'login': 'Login',
        'logoff': 'Logoff',
        'contact': 'Contact',
        'subject': 'Subject',
        'message': 'Message',
        'submit': 'Submit',
        'msg1': 'Please note that you include a contact option in the message if you wish to be contacted..',
        'msg2': 'For data protection reasons, no name or email address is requested!',
        'users': 'Users',
        'analysis': 'Logfile analysis',
        'dropbox': 'Fileupload',
        'check modifications': 'Server File optimisation',
        'edit message: Page is protected': 'Edit message: Page not found',
        'edit message: Page not valid': 'Edit message: Page not valid',
        'edit message: Page not found': 'Edit message: Page not found',
        'edit mail imprint': 'Edit eMail imprint',
        'edit': 'Edit',
        'required': 'Please fill the Field!',
        'top': 'Top',
        'up': 'Up',
        'down': 'Down',
        'bottom': 'Bottom',
        'add': 'Add',
        'delete': 'Delete',
        'move': 'Move',
        'exit': 'Exit',
        'fail': 'Fail',
        'show functions': 'Show funktions',
        'hide functions': 'Hide funktions'
      };
      break;
  }
  var self = this;
  var tb = document.getElementById('toolbar-left');
  if (tb) {
    // login
    this.btn = document.createElement('button');
    this.btn.setAttribute('type', 'button');
    this.btn.className = 'cmd-login';
    this.btn.title = this.translate('login');
    tb.appendChild(this.btn);
    this.btn.addEventListener('click', function (event) { self.show(); event.stopPropagation();});
    this.btnDest = document.createElement('div');
    this.btnDest.setAttribute('id', 'functions');
    tb.appendChild(this.btnDest);
    // contact
    this.contactbtn = document.createElement('button');
    this.contactbtn.setAttribute('type', 'button');
    this.contactbtn.className = 'cmd-contact';
    this.contactbtn.title = this.translate('contact');
    tb.appendChild(this.contactbtn);
    this.contactbtn.addEventListener('click', function (event) { self.showContact(); event.stopPropagation();});
  }
  window.addEventListener('online', function () { self.updateNetworkStatus(); });
  window.addEventListener('offline', function () { self.updateNetworkStatus(); });
  document.addEventListener('visibilitychange', function (event) {
    if (!document.hidden) {
      self.checkSession(true);
    }
  }, false);
  this.updateNetworkStatus();
  this.checkSession();
}

App.prototype = {
  /** translate
   * @param {String} str
   * @returns {String|Array} translation
   */
  translate: function (str) {
    // todo: fill translation array
    if (!this.translations[str]) {
      console.log('app: missing translation: ' + str);
    }
    return (this.translations[str] || str);
  },
  request: function (method, url, responseType, callback, data) {
    var xhr = new XMLHttpRequest();
    var async = typeof callback === "function";
    xhr.open(method, url, async);
    if (async) {
      xhr.responseType = responseType;
      xhr.addEventListener('load', function (event) {
        callback(xhr);
      });
    }
    if ((method == "POST") && data) {
      xhr.setRequestHeader("Content-Type", "application/json; charset=UTF-8");
      xhr.send(JSON.stringify(data));
    } else {
      xhr.send();
    }
  },
  addScript: function (href, dest) {
    var id = href.split('/').pop();
    if (!(dest ? dest.document.getElementById(id) : document.getElementById(id))) {
      var script = (dest ? dest.document.createElement('script') : document.createElement('script'));
      script.id = id;
      script.src = href;
      script.async = false;
      (dest ? dest.document.head.appendChild(script) : document.head.appendChild(script));
    }
  },
  addStyle: function (href, dest) {
    var id = href.split('/').pop();
    if (!(dest ? dest.document.getElementById(id) : document.getElementById(id))) {
      var style = (dest ? dest.document.createElement('link') : document.createElement('link'));
      style.id = id;
      style.type = 'text/css';
      style.rel = 'stylesheet';
      style.href = href;
      (dest ? dest.document.head.appendChild(style) : document.head.appendChild(style));
    }
  },
  popup: function (url, name) {
    window.open(url, '_blank', 'location=no,status=yes,menubar=no,scrollbars=yes,toolbar=no,resizable=yes');
  },
  win: function (url, name) {
    var win = window.open('', '_blank', 'location=no,status=yes,menubar=no,scrollbars=yes,toolbar=no,resizable=yes');
    win.document.open();
    win.document.write(
    '<!doctype html>' +
    '<html>' +
      '<head>' +
        '<meta charset="utf-8">' +
      '</head>' +
      '<body>' +
      '</body>' +
    '</html>');
    win.document.close();
    var title = win.document.createElement('title');
    title.appendChild(win.document.createTextNode(name));
    win.document.head.appendChild(title);
    var meta = win.document.createElement('meta');
    meta.setAttribute('name', 'viewport');
    meta.setAttribute('content', 'width=device-width, initial-scale=1.0');
    win.document.head.appendChild(meta);
    var meta = win.document.createElement('meta');
    meta.setAttribute('name', 'url');
    meta.setAttribute('content', 'https://' + window.location.hostname + url);
    win.document.head.appendChild(meta);
    //win.history.pushState({}, name, 'https://' + window.location.hostname + url);
    var link = win.document.createElement('link');
    link.setAttribute('rel', 'icon');
    link.setAttribute('type', 'image/icon');
    link.setAttribute('href', '/favicon.ico');
    win.document.head.appendChild(link);
    link = win.document.createElement('link');
    link.setAttribute('rel', 'shortcut icon');
    link.setAttribute('type', 'image/x-icon');
    link.setAttribute('href', '/favicon.ico');
    win.document.head.appendChild(link);
    this.addStyle('/cms/cms.css', win);
    return win;
  },
  edit: function (url) {
    var win = this.win(url + '?cmd=edit', 'edit');
    this.addScript('/cms/globals.js', win);
    this.addScript('/cms/calendar.js', win);
    this.addScript('/cms/tabsheets.js', win);
    this.addScript('/cms/file.js', win);
    this.addScript('/cms/abbr.js', win);
    this.addScript('/cms/link.js', win);
    this.addScript('/cms/charmap.js', win);
    this.addScript('/cms/dialog.js', win);
    this.addScript('/cms/contextmenu.js', win);
    this.addScript('/cms/editor.core.js', win);
    this.addScript('/cms/editor.cms.js', win);
    this.addScript('/cms/main.js', win);
  },
  add: function (url) {
    var win = this.win(url + '?cmd=add', 'add');
    this.addScript('/cms/globals.js', win);
    this.addScript('/cms/calendar.js', win);
    this.addScript('/cms/tabsheets.js', win);
    this.addScript('/cms/file.js', win);
    this.addScript('/cms/abbr.js', win);
    this.addScript('/cms/link.js', win);
    this.addScript('/cms/charmap.js', win);
    this.addScript('/cms/dialog.js', win);
    this.addScript('/cms/contextmenu.js', win);
    this.addScript('/cms/editor.core.js', win);
    this.addScript('/cms/editor.cms.js', win);
    this.addScript('/cms/main.js', win);
  },
  delete: function (url) {
    var win = this.win(url + '?cmd=delete', 'delete');
    this.addScript('/cms/globals.js', win);
    this.addScript('/cms/link.js', win);
    this.addScript('/cms/dialog.js', win);
    this.addScript('/cms/main.js', win);
  },
  move: function (url) {
    var win = this.win(url + '?cmd=move', 'move');
    this.addScript('/cms/globals.js', win);
    this.addScript('/cms/link.js', win);
    this.addScript('/cms/dialog.js', win);
    this.addScript('/cms/main.js', win);
  },
  dropbox: function (url) {
    var win = this.win(url + '?cmd=dropbox', 'dropbox');
    this.addScript('/cms/globals.js', win);
    this.addScript('/cms/file.js', win);
    this.addScript('/cms/dialog.js', win);
    this.addScript('/cms/main.js', win);
  },
  analyze: function (url) {
    var win = this.win(url, 'analyze');
    this.addScript('/cms/globals.js', win);
    this.addScript('/cms/analyze.js', win);
  },
  users: function (url) {
    var win = this.win(url + '?cmd=users', 'users');
    this.addScript('/cms/tabsheets.js', win);
    //this.addScript('/cms/app.js', win);
    this.addScript('/cms/dialog.js', win);
    this.addScript('/cms/main.js', win);
  },
  user: function (url, title) {
    var win = this.win(url, title);
    this.addScript('/cms/globals.js', win);
    this.addScript('/cms/calendar.js', win);
    this.addScript('/cms/tabsheets.js', win);
    this.addScript('/cms/link.js', win);
    this.addScript('/cms/dialog.js', win);
    this.addScript('/cms/main.js', win);
  },
  addButton: function (type, className, title, click, dest) {
    var el;
    if (dest) {
      el = dest.querySelector('button.' + className);
    } else {
      el = this.btnDest.querySelector('button.' + className);
    }
    if (!el) {
      el = document.createElement('button');
      el.setAttribute('type', 'button');
      el.setAttribute('data-type', type);
      el.className = className;
      el.title = this.translate(title);
      el.addEventListener('click', click);
      if (dest) {
        dest.appendChild(el);
      } else {
        this.btnDest.appendChild(el);
      }
    }
    return el;
  },
  deleteMessage: function (msgid) {
    var self = this;
    this.request('GET', window.location.pathname + '?cmd=delmsg&msgid=' + msgid, 'json', function (xhr) {
      if (xhr.status == 200) {
        if (xhr.response.result !== 'deleted') {
          alert(self.translate(xhr.response.result));
        } else {
          var msg = document.querySelector('.message[data-msgid="' + xhr.response.msgid + '"]');
          if (msg) {
            msg.parentNode.removeChild(msg);
          }
        }
      } else {
        alert(self.translate('fail'));
      }
    });
  },
  updateStatus: function () {
    var self = this;
    if (localStorage.getItem('__Host-rights') !== 'none') {
      if (this.btn) {
        this.btn.classList.add('active');
        this.btn.setAttribute('title', this.translate('logoff'));
        var url = window.location.pathname;
        if ((localStorage.getItem('__Host-rights') === 'redakteur') || (localStorage.getItem('__Host-rights') === 'admin')) {
          if (!document.getElementById('cms.css')) {
            var style = document.createElement("link");
            style.type = 'text/css';
            var isRunLocally = document.URL.toLowerCase().search(/^file:/) != -1;
            style.href = (isRunLocally ? '' : '/') + 'styles/cms.css';
            style.id = 'cms.css';
            style.rel = 'stylesheet';
            document.head.appendChild(style);
          }
        }
      }
      if (localStorage.getItem('__Host-rights') === 'admin') {
        var tb = document.getElementById('toolbar-right');
        var search = document.getElementById('searchfrm');
        this.addButton('admin-btn', 'func-btn', 'show functions', function (event) {
          this.classList.toggle('active');
          if (self.btnDest) {
            self.btnDest.classList.toggle('active');
          }
          if (search) {
            search.classList.toggle('inactive');
          }
          this.setAttribute('title', this.classList.contains('active') ? self.translate('hide functions') : self.translate('show functions'));
        }, tb);
        // add admin buttons
// this.popup(url + '?cmd=config', 'cmd-config', translate('configuration'), 'config');
        this.addButton('admin-btn', 'cmd-user', 'users', function (event) {
          self.users('/');
        });
        this.addButton('admin-btn', 'cmd-analyze', 'analysis', function (event) {
          self.analyze('/');
        });
        this.addButton('admin-btn', 'cmd-dropbox', 'dropbox', function (event) {
          self.dropbox('/');
        });
// this.popup(url + '?cmd=abbreviation', 'cmd-abbreviation', translate('abbreviations'), 'abbreviations');
// this.popup(url + '?cmd=checklist', 'cmd-checklist', 'Checkliste', 'checklist');
        this.addButton('admin-btn', 'cmd-protected', 'edit message: Page is protected', function (event) {
          self.edit('/protected');
        });
        this.addButton('admin-btn', 'cmd-notvalid', 'edit message: Page not valid', function (event) {
          self.edit('/notValid');
        });
        this.addButton('admin-btn', 'cmd-notfound', 'edit message: Page not found', function (event) {
          self.edit('/notFound');
        });
        this.addButton('admin-btn', 'cmd-imprint', 'edit mail imprint', function (event) {
          self.edit('/emailImprint');
        });
        this.addButton('admin-btn', 'cmd-refresh', 'check modifications', function (event) {
          self.request('GET', '/?cmd=modification');
        });
      }
      if ((localStorage.getItem('__Host-rights') === 'redakteur') || (localStorage.getItem('__Host-rights') === 'admin')) {
        // add redakteur buttons
        var article = document.querySelector('article');
        if (article) {
          var div = article.querySelector('div.cms-buttons');
          if (!div) {
            div = document.createElement('div');
            div.className = 'cms-buttons';
            article.insertBefore(div, article.firstChild);
          }
          this.addButton('cms-btn', 'cmd-edit', 'edit', function (event) {
            self.edit(url);
          }, div);
          var flags = article.getAttribute('data-flag');
          if (!flags || flags != 'system') {
            this.addButton('cms-btn', 'cmd-remove', 'delete', function (event) {
              self.delete(url);
            }, div);
            this.addButton('cms-btn', 'cmd-move', 'move', function (event) {
              self.move(url);
            }, div);
          }
        }
        var blog = document.getElementById('blog_plugin');
        if (blog) {
          div = document.createElement('div');
          div.className = 'cms-buttons';
          blog.parentNode.insertBefore(div, blog);
          this.addButton('cms-btn', 'cmd-add', 'add', function (event) {
            self.add(url);
          }, div);
          var blogItems = blog.querySelectorAll('div.blogItem');
          var len = blogItems.length, div;
          for (var i=0; i<len; i++) {
            if (blogItems[i].getAttribute('data-href')) {
              var div = blogItems[i].querySelector('div.cms-buttons');
              if (!div) {
                div = document.createElement('div');
                div.className = 'cms-buttons';
                blogItems[i].insertBefore(div, blogItems[i].firstChild);
              }
              this.addButton('cms-btn', 'cmd-edit', 'edit', function (event) {
                var href = this.parentNode.parentNode.getAttribute('data-href');
                self.edit(href);
              }, div);
              this.addButton('cms-btn', 'cmd-remove', 'delete', function (event) {
                var href = this.parentNode.parentNode.getAttribute('data-href');
                self.delete(href, 'delete');
              }, div);
            }
          }
        }
        var ul = document.querySelector('nav#content-nav > ul');
        if (!ul) {
          let nav = document.querySelector('nav#content');
          if (!nav) {
            nav = document.createElement('nav');
            nav.id = 'content-nav';
            let main = document.querySelector('main#content');
            main.appendChild(nav);
          }
          ul = document.createElement('ul');
          nav.appendChild(ul);
        }
        if (ul) {
          var li = ul.children, len = li.length, div, link, href;
          for (var i=0; i<len; i++) {
            link = li[i].getElementsByTagName('a')[0];
            if (link) {
              href = link.getAttribute('href');
              var div = li[i].querySelector('div.cms-buttons');
              if (!div) {
                div = document.createElement('div');
                div.className = 'cms-buttons';
                li[i].insertBefore(div, li[i].firstChild);
              }
              div.setAttribute('data-href', href);
              if (i>0) {
                this.addButton('cms-btn', 'cmd-top', 'top', function (event) {
                  var href = this.parentNode.getAttribute('data-href');
                  self.request('GET', href + '?cmd=top', 'json', function (xhr) {
                    location.reload();
                  });
                }, div);
                this.addButton('cms-btn', 'cmd-up', 'up', function (event) {
                  var href = this.parentNode.getAttribute('data-href');
                  self.request('GET', href + '?cmd=up', 'json', function (xhr) {
                    location.reload();
                  });
                }, div);
              }
              if (i < len-1) {
                this.addButton('cms-btn', 'cmd-down', 'down', function (event) {
                  var href = this.parentNode.getAttribute('data-href');
                  self.request('GET', href + '?cmd=down', 'json', function (xhr) {
                    location.reload();
                  });
                }, div);
                this.addButton('cms-btn', 'cmd-bottom', 'bottom', function (event) {
                  var href = this.parentNode.getAttribute('data-href');
                  self.request('GET', href + '?cmd=bottom', 'json', function (xhr) {
                    location.reload();
                  });
                }, div);
              }
              this.addButton('cms-btn', 'cmd-move', 'move', function (event) {
                var href = this.parentNode.getAttribute('data-href');
                self.move(href);
              }, div);
              this.addButton('cms-btn', 'cmd-edit', 'edit', function (event) {
                var href = this.parentNode.getAttribute('data-href');
                self.edit(href);
              }, div);
            }
          }
          if (!ul.querySelector('button.cmd-add')) {
            li = document.createElement('li');
            li.setAttribute('data-type', 'cms-btn');
            ul.appendChild(li);
            div = document.createElement('div');
            div.className = 'cms-buttons';
            div.setAttribute('data-href', window.location.pathname);
            li.appendChild(div);
            this.addButton('cms-btn', 'cmd-add', 'add', function (event) {
              var href = this.parentNode.getAttribute('data-href');
              self.add(href);
            }, div);
          }
        }
        var messages = document.querySelectorAll('.message'), len = messages.length, i;
        for (i = 0; i < len; i++) {
          var h = messages[i].querySelector('h5');
          var div = document.createElement('div');
          div.className = 'cms-buttons';
          messages[i].insertBefore(div, h);
          if (h) {
            var el = this.addButton('cms-btn', 'cmd-remove', 'delete', function (event) {
              var msgid = this.getAttribute('data-msgid');
              self.deleteMessage(msgid);
            }, div);
            el.setAttribute('data-msgid', messages[i].getAttribute('data-msgid'));
          }
        }
      }
    } else {
      if (this.btn) {
        this.btn.classList.remove('active');
        this.btn.setAttribute('title', this.translate('login'));
      }
      var search = document.getElementById('searchfrm');
      if (search) {
        search.classList.remove('inactive');
      }
    }
    var els, len, i;
    if (localStorage.getItem('__Host-rights') !== 'admin') {
      els = document.querySelectorAll('[data-type="admin-btn"]');
      for (i = 0, len = els.length; i < len; i++) {
        els[i].parentNode.removeChild(els[i]);
      }
    }
    if ((localStorage.getItem('__Host-rights') !== 'redakteur') && (localStorage.getItem('__Host-rights') !== 'admin')) {
      els = document.querySelectorAll('div.cms-buttons');
      for (i = 0, len = els.length; i < len; i++) {
        els[i].parentNode.removeChild(els[i]);
      }
      els = document.querySelectorAll('[data-type="cms-btn"]');
      for (i = 0, len = els.length; i < len; i++) {
        els[i].parentNode.removeChild(els[i]);
      }
    }
  },
  updateNetworkStatus: function () {
    if (navigator.onLine) {
      document.body.classList.remove('offline');
      document.body.classList.add('online');
    } else {
      document.body.classList.remove('online');
      document.body.classList.add('offline');
    }
  },
  generateUUID: function () {
    var d = new Date().getTime();
    var uuid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {
      var r = (d + Math.random()*16)%16 | 0;
      d = Math.floor(d/16);
      return (c=='x' ? r : (r&0x3|0x8)).toString(16);
    });
    return uuid;
  },
  sha256: function (s) {
    var chrsz = 8;
    var hexcase = 1;

    function safe_add (x, y) {
      var lsw = (x & 0xFFFF) + (y & 0xFFFF);
      var msw = (x >> 16) + (y >> 16) + (lsw >> 16);
      return (msw << 16) | (lsw & 0xFFFF);
    };

    function S (X, n) { return ( X >>> n ) | (X << (32 - n)); };
    function R (X, n) { return ( X >>> n ); };
    function Ch(x, y, z) { return ((x & y) ^ ((~x) & z)); };
    function Maj(x, y, z) { return ((x & y) ^ (x & z) ^ (y & z)); };
    function Sigma0256(x) { return (S(x, 2) ^ S(x, 13) ^ S(x, 22)); };
    function Sigma1256(x) { return (S(x, 6) ^ S(x, 11) ^ S(x, 25)); };
    function Gamma0256(x) { return (S(x, 7) ^ S(x, 18) ^ R(x, 3)); };
    function Gamma1256(x) { return (S(x, 17) ^ S(x, 19) ^ R(x, 10)); };

    function core_sha256 (m, l) {
      var K = new Array(0x428A2F98, 0x71374491, 0xB5C0FBCF, 0xE9B5DBA5, 0x3956C25B, 0x59F111F1, 0x923F82A4, 0xAB1C5ED5, 0xD807AA98, 0x12835B01, 0x243185BE, 0x550C7DC3, 0x72BE5D74, 0x80DEB1FE, 0x9BDC06A7, 0xC19BF174, 0xE49B69C1, 0xEFBE4786, 0xFC19DC6, 0x240CA1CC, 0x2DE92C6F, 0x4A7484AA, 0x5CB0A9DC, 0x76F988DA, 0x983E5152, 0xA831C66D, 0xB00327C8, 0xBF597FC7, 0xC6E00BF3, 0xD5A79147, 0x6CA6351, 0x14292967, 0x27B70A85, 0x2E1B2138, 0x4D2C6DFC, 0x53380D13, 0x650A7354, 0x766A0ABB, 0x81C2C92E, 0x92722C85, 0xA2BFE8A1, 0xA81A664B, 0xC24B8B70, 0xC76C51A3, 0xD192E819, 0xD6990624, 0xF40E3585, 0x106AA070, 0x19A4C116, 0x1E376C08, 0x2748774C, 0x34B0BCB5, 0x391C0CB3, 0x4ED8AA4A, 0x5B9CCA4F, 0x682E6FF3, 0x748F82EE, 0x78A5636F, 0x84C87814, 0x8CC70208, 0x90BEFFFA, 0xA4506CEB, 0xBEF9A3F7, 0xC67178F2);
      var HASH = new Array(0x6A09E667, 0xBB67AE85, 0x3C6EF372, 0xA54FF53A, 0x510E527F, 0x9B05688C, 0x1F83D9AB, 0x5BE0CD19);
      var W = new Array(64);
      var a, b, c, d, e, f, g, h, i, j;
      var T1, T2;

      m[l >> 5] |= 0x80 << (24 - l % 32);
      m[((l + 64 >> 9) << 4) + 15] = l;

      for ( var i = 0; i<m.length; i+=16 ) {
        a = HASH[0];
        b = HASH[1];
        c = HASH[2];
        d = HASH[3];
        e = HASH[4];
        f = HASH[5];
        g = HASH[6];
        h = HASH[7];

        for ( var j = 0; j<64; j++) {
          if (j < 16) W[j] = m[j + i];
          else W[j] = safe_add(safe_add(safe_add(Gamma1256(W[j - 2]), W[j - 7]), Gamma0256(W[j - 15])), W[j - 16]);

          T1 = safe_add(safe_add(safe_add(safe_add(h, Sigma1256(e)), Ch(e, f, g)), K[j]), W[j]);
          T2 = safe_add(Sigma0256(a), Maj(a, b, c));

          h = g;
          g = f;
          f = e;
          e = safe_add(d, T1);
          d = c;
          c = b;
          b = a;
          a = safe_add(T1, T2);
        };

        HASH[0] = safe_add(a, HASH[0]);
        HASH[1] = safe_add(b, HASH[1]);
        HASH[2] = safe_add(c, HASH[2]);
        HASH[3] = safe_add(d, HASH[3]);
        HASH[4] = safe_add(e, HASH[4]);
        HASH[5] = safe_add(f, HASH[5]);
        HASH[6] = safe_add(g, HASH[6]);
        HASH[7] = safe_add(h, HASH[7]);
      };
      return HASH;
    };

    function str2binb (str) {
      var bin = Array();
      var mask = (1 << chrsz) - 1;
      for(var i = 0; i < str.length * chrsz; i += chrsz) {
        bin[i>>5] |= (str.charCodeAt(i / chrsz) & mask) << (24 - i%32);
      };
      return bin;
    };

    function Utf8Encode(string) {
      string = string.replace(/\r\n/g,"\n");
      var utftext = "";

      for (var n = 0, len = string.length; n < len; n++) {
        var c = string.charCodeAt(n);
        if (c < 128) {
          utftext += String.fromCharCode(c);
        } else if((c > 127) && (c < 2048)) {
          utftext += String.fromCharCode((c >> 6) | 192);
          utftext += String.fromCharCode((c & 63) | 128);
        } else {
          utftext += String.fromCharCode((c >> 12) | 224);
          utftext += String.fromCharCode(((c >> 6) & 63) | 128);
          utftext += String.fromCharCode((c & 63) | 128);
        }
      };
      return utftext;
    };

    function binb2hex (binarray) {
      var hex_tab = hexcase ? "0123456789ABCDEF" : "0123456789abcdef";
      var str = "";
      for(var i = 0, len = binarray.length * 4; i < len; i++) {
        str += hex_tab.charAt((binarray[i>>2] >> ((3 - i%4)*8+4)) & 0xF) + hex_tab.charAt((binarray[i>>2] >> ((3 - i%4)*8  )) & 0xF);
      };
      return str;
    };

    s = Utf8Encode(s);
    return binb2hex(core_sha256(str2binb(s), s.length * chrsz));
  },
  checkSession: function (show) {
    show = show || false;
    var self = this;
    // get SessionID
    var SessionID = '';
    const value = document.cookie;
    SessionID = value.split('__Host-SessionID=');
    if (SessionID.length === 2) {
      SessionID = SessionID.pop().split(';').shift();
    }
    // check if request is cached
    var isCached = false;
    var navigation = performance.getEntriesByType("navigation");
    if (navigation && navigation.length > 0) {
      isCached = navigation[0].transferSize === 0;
    }
    if ((SessionID != localStorage.getItem('__Host-SessionID')) || isCached || show) {
      // on new session sent screen/display values
      // on cached content revalidate status
      this.request("POST", "/xhr.json", 'json',
        function (xhr) {
          if (xhr.status === 200) {
            localStorage.setItem('__Host-SessionID', xhr.response['__Host-SessionID']);
            localStorage.setItem('__Host-rights', xhr.response['__Host-rights']);
          }
          self.updateStatus();
        },
        {
          width: screen.width,
          height: screen.height,
          density: ((typeof(window.devicePixelRatio) != "undefined") ? window.devicePixelRatio : 1)
        });
    } else {
      self.updateStatus();
    }
  },
  updateUserStatus: function (xhr) {
    if (xhr.status == 200) {
      localStorage.setItem('__Host-rights', xhr.response['__Host-rights']);
      this.updateStatus();
      this.close();
    } else {
      alert(this.translate('fail'));
    }
  },
  login: function () {
    var self = this, username = this.username.value, password = this.password.value;
    if (username && password) {
      var uuid = this.generateUUID();
      this.request("POST", window.location.pathname + '?cmd=login', 'json', function (xhr) {
        var sha = self.sha256(xhr.response['secret'] + username + ':' + uuid + password);
        if (xhr.status == 200) {
          self.request("POST", window.location.pathname + '?cmd=login', 'json', function (xhr2) {
            self.updateUserStatus(xhr2);
          },
          { user: username, data: sha });
        }
      },
      { user: username, identifier: uuid });
    }
  },
  logoff: function () {
    var self = this;
    this.request("GET", window.location.pathname + '?cmd=logoff', 'json', function (xhr) {
      self.updateUserStatus(xhr);
    });
  },
  showLogin: function () {
    var self = this;
    this.dlg = document.createElement('div');
    this.dlg.className = 'communication';
    var form = document.createElement('form');
    this.dlg.appendChild(form);
    var labelName = document.createElement('label');
    form.appendChild(labelName);
    this.username = document.createElement('input');
    this.username.setAttribute('type', 'text');
    this.username.setAttribute('id', 'userName');
    this.username.setAttribute('name', 'userName');
    this.username.setAttribute('title', this.translate('username'));
    this.username.setAttribute('placeholder', this.translate('username'));
    this.username.setAttribute('maxlength', '32');
    this.username.setAttribute('autocapitalize', 'off');
    this.username.setAttribute('autocomplete', 'off');
    this.username.setAttribute('spellcheck', 'false');
    this.username.setAttribute('autocorrect', 'off');
    form.appendChild(this.username);
    labelName.setAttribute('for', 'userName');
    labelName.appendChild(document.createTextNode(this.translate('username')));
    var labelPsw = document.createElement('label');
    form.appendChild(labelPsw);
    this.password = document.createElement('input');
    this.password.setAttribute('type', 'password');
    this.password.setAttribute('id', 'userPsw');
    this.password.setAttribute('name', 'userPsw');
    this.password.setAttribute('title', this.translate('password'));
    this.password.setAttribute('placeholder', this.translate('password'));
    this.password.setAttribute('maxlength', '32');
    this.password.setAttribute('spellcheck', 'false');
    this.password.setAttribute('autocomplete', 'off');
    if ('autocapitalize' in this.password) {
      this.password.setAttribute("autocapitalize", "none");
    };
    if ('autocorrect' in this.password) {
      this.password.setAttribute("autocorrect", "none");
    };
    form.appendChild(this.password);
    labelPsw.setAttribute('for', 'userPsw');
    labelPsw.appendChild(document.createTextNode(this.translate('password')));
    var btn = document.createElement('button');
    form.appendChild(btn);
    btn.setAttribute('type', 'button');
    btn.appendChild(document.createTextNode(this.translate('login')));
    btn.addEventListener('click', function (event) {
      self.login();
      event.stopPropagation();
    });
    this.password.addEventListener('keyup', function(event) {
      if (event.code === 'Enter') {
        event.preventDefault();
        self.login();
      }
    });
    document.body.appendChild(this.dlg);
    this.username.focus();
    this.bg.addEventListener('click', function() { self.close(); });
  },
  showLogoff: function () {
    var self = this;
    this.dlg = document.createElement('div');
    this.dlg.className = 'communication';
    var form = document.createElement('form');
    this.dlg.appendChild(form);
    var btn = document.createElement('button');
    form.appendChild(btn);
    btn.setAttribute('type', 'button');
    btn.setAttribute('id', 'logoffBtn');
    btn.addEventListener('click', function (event) {
      localStorage.setItem('__Host-rights', 'none');
      self.logoff();
      event.stopPropagation();
    });
    btn.appendChild(document.createTextNode(this.translate('logoff')));
    document.body.appendChild(this.dlg);
    btn.focus();
  },
  validate: function (form) {
    var native_required = 'required' in document.createElement('input');
    var values = [];
    var els = form.elements;
    var el, tagName, type;
    for (var i = 0; i < els.length; i++) {
      if (els[i].name == 'spam') {
        els[i].setAttribute('value', 'falsch');
      }
      el = els[i];
      tagName = el.tagName.toLowerCase();
      if (tagName == 'input') {
        type = el.type.toLowerCase();
        if ((type == 'radio') || (type == 'checkbox')) {
          var radioLength = el.length;
          if(radioLength == undefined) {
            if(el.checked) {
              values[el.name] = el.value;
            } else {
              values[el.name] = '';
            }
          } else {
            for(var i = 0; i < radioLength; i++) {
              if(el[i].checked) {
                values[el.name][i] = el[i].value;
              }
            }
          }
        } else if (type == 'file') {
          values[el.name] = el.value.split('\\').pop().split('/').pop();
          el = document.getElementById(el.name + '-src');
          if (el) {
            values['src'] = el.value;
          }
        } else {
          values[el.name] = el.value;
        }
      } else if (tagName == 'select') {
        values[el.name] = el[el.selectedIndex].value;
      } else if (tagName == 'textarea') {
        values[el.name] = el.value;
      } else if (el.name != '') { // buttons etc.
        values[el.name] = el.value;
      }
      if (!native_required && (el.getAttribute('required') == 'required') && (values[el.name].length < 2)) {
        alert(this.translate('required'));
        el.focus();
        return false;
      }
    }
    return true;
  },
  showContact: function () {
    if (localStorage.getItem('__Host-rights') === 'admin') {
      window.location.href = '/Kontakt';
    } else {
      if (this.dlg) {
        this.close();
      }
      var self = this;
      this.dlg = document.createElement('div');
      this.dlg.className = 'communication';
      var form = document.createElement('form');
      form.setAttribute('method', 'POST');
      form.setAttribute('enctype', 'multipart/form-data');
      form.setAttribute('action', '/Kontakt');
      var spam = document.createElement('input');
      spam.setAttribute('type', 'hidden');
      spam.setAttribute('name', 'spam');
      spam.setAttribute('id', 'spam');
      spam.setAttribute('value', 'true');
      form.appendChild(spam);
      this.dlg.appendChild(form);
      var labelSubject = document.createElement('label');
      form.appendChild(labelSubject);
      var subject = document.createElement('input');
      subject.setAttribute('type', 'text');
      subject.setAttribute('id', 'subject');
      subject.setAttribute('name', 'subject');
      subject.setAttribute('title', this.translate('subject'));
      subject.setAttribute('placeholder', this.translate('subject'));
      subject.setAttribute('required', 'required');
      form.appendChild(subject);
      labelSubject.setAttribute('for', 'subject');
      labelSubject.appendChild(document.createTextNode(this.translate('subject')));
      var labelMessage = document.createElement('label');
      form.appendChild(labelMessage);
      var message = document.createElement('textarea');
      message.setAttribute('id', 'message');
      message.setAttribute('name', 'message');
      message.setAttribute('title', this.translate('message'));
      message.setAttribute('placeholder', this.translate('message'));
      message.setAttribute('required', 'required');
      form.appendChild(message);
      labelMessage.setAttribute('for', 'message');
      labelMessage.appendChild(document.createTextNode(this.translate('message')));
      var msg = document.createElement('p');
      msg.appendChild(document.createTextNode(this.translate('msg1')));
      msg.appendChild(document.createElement('br'));
      msg.appendChild(document.createTextNode(this.translate('msg2')));
      form.appendChild(msg);
      var btn = document.createElement('button');
      form.appendChild(btn);
      btn.setAttribute('type', 'submit');
      btn.appendChild(document.createTextNode(this.translate('submit')));
      btn.addEventListener('click', function (event) {
        if (!self.validate(form)) {
          event.stopPropagation();
        }
      });
      document.body.appendChild(this.dlg);
      subject.focus();
      this.bg.addEventListener('click', function() { self.close(); });
    }
  },
  close: function () {
    var self = this;
    if (this.dlg) {
      document.body.removeChild(this.dlg);
      this.dlg = null;
    }
    this.bg.removeEventListener('click', function() { self.close(); });
  },
  show: function () {
    if (this.dlg) {
      this.close();
    }
    var self = this;
    if (this.btn) {
      if (this.btn.classList.contains('active')){
        this.showLogoff();
      } else {
        this.showLogin();
      }
    }
    this.bg.addEventListener('click', function() { self.close(); });
  }
};

// create app
new App();

})();
